#include <linux/joystick.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/select.h>
#include <unistd.h>
#include "joy.h"

#define EV(x,y)  (((x)<<8)|(y))



JOY* JOYopen(char *name)
{
  int joyfd;
  char naxes, nbuttons;
  int i;
  JOY *J;
  joyfd= open(name, O_RDONLY);
  if (joyfd<0) return NULL;
  if (ioctl(joyfd,JSIOCGAXES,&naxes)<0) return NULL;
  if (ioctl(joyfd,JSIOCGBUTTONS,&nbuttons)<0) return NULL;
  J= calloc(1,sizeof(*J));
  J->fd= joyfd;
  J->naxes= naxes;
  J->axes= calloc(naxes, sizeof(J->axes[0]));
  J->nbuttons= nbuttons;
  J->buttons= calloc(nbuttons, sizeof(J->buttons[0]));
  J->amap= malloc(sizeof(J->amap[0])*naxes);
  for(i=0;i<naxes;i++) J->amap[i]= i;
  J->bmap= malloc(sizeof(J->bmap[0])*nbuttons);
  for(i=0;i<nbuttons;i++) J->bmap[i]= i;
  if (ioctl(joyfd,JSIOCGNAME(sizeof(J->name)),J->name)<0)
    strcpy(J->name,"Unknown");
  else
    J->name[sizeof(J->name)-1]= 0;
  return J; 
}

int JOYread(JOY *J)
{
  struct js_event event;  
  int R,nr;

  R= read(J->fd, &event, sizeof(event));
  if (R!=sizeof(event))
    return -1;
  event.type &= ~JS_EVENT_INIT;
  if (event.type==JS_EVENT_AXIS)
  {
    if (event.number>=J->naxes) return -1;
    nr= J->amap[event.number];
    if (nr<0 || nr>=J->naxes) return -1;
    J->axes[nr]= event.value;
    return EV(JOYm, nr);
  }
  if (event.number>=J->nbuttons) return -1;
  nr= J->bmap[event.number];
  if (nr<0 || nr>=J->nbuttons) return -1;
  J->buttons[nr]= event.value;
  if (event.value) return EV(JOYp, nr);
                     else EV(JOYr, nr);
}

void JOYclose(JOY *J)
{
  free(J->buttons);
  free(J->axes);
  free(J->amap);
  free(J->bmap);
  close(J->fd);
  free(J);
}

