#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "jview.h"
#include "proto.h"
#include "gfx.h"
#include "ticktock.h"
#include "joy.h"


static int servsock=-1, datasock=-1;
static unsigned int pixW, pixH;
static jview_t *jv;


void init_pixgrid_jview()
{
  int bpl, bpp, order[3],i;
  unsigned char *buffer;
  char fmt[5];
  int dummy;

  gfx_pixgrid_attribs(Wpix, &dummy, &pixW, &pixH);
  gfx_pixgrid_buffer(Wpix,&bpl,&bpp,order,&buffer);
 
  for(i=0;i<4;i++) fmt[i]= 'X';
  fmt[4]= 0;
  fmt[order[0]]= 'R'; fmt[order[1]]= 'G'; fmt[order[2]]= 'B';

  jv= jview_init(fmt,buffer,pixW,pixH,bpl, 640, 480);
}

void have_image(uint32_t Limage,unsigned char* image) 
{
  jview_decode(jv, image, Limage);
  gfx_pixgrid_update(Wpix, 0,0, pixW, pixH);
} 

void message(char *msg)
{
  printf("ROBOT: %s\n",msg);
}


static char joyX, joyY;

static char joyconvert(int V)
{
  int S;
  if (V<0) { V= -V; S= 0x80; }
      else { S= 0; }
  if (V>32767) V= 32767;
  if (V>8192) V= 0x0f;
         else V= 0;
  // V >>= 11; 
  return S | V;
}

int main()
{
  int fd;
  JOY *joy;
  gfx_init(NULL,650,525,"Remote Camera");
  gfx_SCREEN_0();
  init_pixgrid_jview();
  servsock= IPFserver(4598);
  if (servsock<0) exit(fprintf(stderr,"can't create server\n"));
  gfx_monitor_file(servsock,100,1, 0,0);
  fd= ticktock_begin(20);
  if (fd>=0)
    gfx_monitor_file(fd,300,1, 0,0);
  joy= JOYopen("/dev/input/js0");
  if (!joy)
    fprintf(stderr,"can't open /dev/input/js0: %s\n", strerror(errno));
  gfx_monitor_file(joy->fd, 400, 1, 0,0); 
  while(1)
  {
    gfx_event_t event;
    if (!gfx_event(NULL,&event)) continue;
    switch(event.type)
    {
    case Wstart: start_video_capture(); break;
    case Wstop: stop_video_capture(); break;
    case Wrecycle: recycle_drive_enable(); break;
    case GFXEV_READY_READ:
      if (event.key==100)
      {
        printf("new connection\n");
        datasock= IPFaccept();
        if (datasock>=0)
        {
          gfx_stop_monitor_file(servsock,100);
          gfx_monitor_file(datasock,200,1, 0,1);
        }
        break;
      }
      if (event.key==200) { IPFread(); break; }
      if (event.key==300) { ticktock_ack(); motor_control(joyX, joyY); break; }
      if (event.key==400)
      {
        switch(JOYread(joy))
        {
        case JOYEV(m,0): joyX= joyconvert(joy->axes[0]); break;
        case JOYEV(m,1): joyY= joyconvert(-joy->axes[1]); break;
        }
        break;
      }
      break;
    case GFXEV_READY_EXCEPT:
      if (event.key==200)
      {
         printf("connection lost\n");
         gfx_stop_monitor_file(datasock,200);
         gfx_monitor_file(servsock,100,1, 0,0);
         break;
      }
      break;
    case GFXEV_QUIT: 
      goto done;
    }
  }
done:
  return 0;
}
