#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include "gpio.h"

static int exported[40];
static int direction[40];
static int valuefd[40];
static int init;

#define DIR_O  1
#define DIR_I  2
#define DIR_U  -1


#define S_EXPORT      "/sys/class/gpio/export"
#define S_UNEXPORT    "/sys/class/gpio/unexport"
#define S_DIRECTION   "/sys/class/gpio/gpio%d/direction"
#define S_VALUE       "/sys/class/gpio/gpio%d/value"

static int ERR(const char *fmt,...)
{
  va_list ap;
  fprintf(stderr,"errno= %d (%s):", errno, strerror(errno));
  va_start(ap,fmt);
  vfprintf(stderr,fmt,ap);
  va_end(ap);
  return -1;
}

static void gpio_init()
{
  int i;
  if (init) return ;
  for(i=0;i<40;i++)
  {
    exported[i]= 0; direction[i]= DIR_U; valuefd[i]= -1;
  }
  init= 1;
}

static int export(int P)
{
  int fd;
  char buf[20];
  if (P<0 || P>=40) return -1;
  if (exported[P]) return 0;
  sprintf(buf,"%d\n",P);
  fd= open(S_EXPORT,O_WRONLY);
  if (fd<0)
    return ERR("can't open export file for pin %d\n",P);
  if (write(fd, buf, strlen(buf))!=strlen(buf))
    { close(fd); return ERR("can't write to export file for pin %d\n",P); }
  close(fd);
  exported[P]= 1;
  return 0;
}

static int unexport(int P)
{
  int fd;
  char buf[20];
  if (P<0 || P>=40) return -1;
  if (!exported[P]) return 0;
  sprintf(buf,"%d\n",P);
  fd= open(S_UNEXPORT,O_WRONLY);
  if (fd<0)
    return ERR("can't open unexport file for pin %d\n",P);
  if (write(fd, buf, strlen(buf))!=strlen(buf))
    { close(fd); return ERR("can't write to unexport file for pin %d\n",P); }
  close(fd);
  exported[P]= 0;
  return 0;
}

static int gpio_dir(int P, char *val, int dir)
{
  int fd; int i;
  char buf[200];
  gpio_init();
  if (export(P)) return -1;
  if (direction[P]==dir) return 0;
  if (valuefd[P]>=0) { close(valuefd[P]); valuefd[P]= -1; }
  sprintf(buf,S_DIRECTION,P);
  for(i=0;i<2;i++)
  {
    fd= open(buf,O_RDWR);
    if (fd<0) ERR("can't open direction file (%s) for pin %d\n",buf,P);
    else break;
    sleep(1); // I don't know man 
  }
  if (fd<0) return -1;
  if (write(fd,val,strlen(val))!=strlen(val))
    { close(fd); return ERR("can't write to direction file for pin %d\n",P); }
  close(fd);
  direction[P]= dir;
  return 0;
}

int gpio_input(int P) { return gpio_dir(P,"in\n",DIR_I); }
int gpio_output(int P) { return gpio_dir(P,"out\n",DIR_O); }

static int gpio_setval(int P, char *V)
{
  char buf[200];
  if (gpio_output(P)) return -1;
  if (valuefd[P]==-1)
  {
    sprintf(buf,S_VALUE,P);
    valuefd[P]= open(buf,O_WRONLY);
    if (valuefd[P]<0) 
      return ERR("can't open value file for pin %d\n",P);
  }
  if (write(valuefd[P], V,2)!=2)
  {
    int eno;
    eno= errno;
    close(valuefd[P]);
    valuefd[P]= -1;
    errno= eno;
    return ERR("can't write to value file for pin %d\n",P);
  }
  return 0;
}

int gpio_set(int P) { return gpio_setval(P, "1\n"); }
int gpio_clr(int P) { return gpio_setval(P, "0\n"); }
int gpio_setiff(int P, int V) { return gpio_setval(P, V? "1\n":"0\n"); }

int gpio_get(int P, int *V)
{
  char buf[200];
  char vv[2];
  if (gpio_input(P)) return -1;
  if (valuefd[P]==-1)
  {
    sprintf(buf,S_VALUE,P);
    valuefd[P]= open(buf,O_RDONLY);
    if (valuefd[P]<0) 
      return ERR("can't open value file for pin %d\n",P);
  }
  if (read(valuefd[P], vv,2)!=2)
  {
    int eno;
    eno= errno;
    close(valuefd[P]);
    valuefd[P]= -1;
    errno= eno;
    return ERR("can't read from value file for pin %d\n",P);
  }
  if (vv[0]=='0') *V= 0;
  else if (vv[0]=='1') *V= 1;
  else return ERR("bad value %02x from value file for pin %d\n",vv[0],P);
  return 0;
}

void gpio_quit()
{
  int i;
  for(i=0;i<40;i++)
  {
    if (valuefd[i]>=0)
    {
      close(valuefd[i]);
      valuefd[i]= -1;
    }
    direction[i]= DIR_U;
  }
  for(i=0;i<40;i++)
   unexport(i);
  init= 0;
}
