#include <stdint.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include "vica.h"
#include "proto.h"
#include "unisel.h"
#include "gpio.h"
#include "spi-rpi.h"


static void remoprint(const char *fmt, va_list ap)
{
  char buf[2000];
  vsnprintf(buf,sizeof(buf),fmt,ap);
  buf[1999]= 0;
  message(buf);
}

static vica_t *vic;
static int datasock;
static SEL *sel;
static int vfd;

void start_video_capture()
{
  printf("Start capture\n");
  if (!vic)
  {
    message("capture device fail, can't start");
    return ;
  }
  vic= vica_start(vic);
  if (!vic)
  { message("capture device fail 2, can't start"); return ; }
  SELadd(sel,vfd,200,SELr);
}


void stop_video_capture()
{
  printf("Stop capture\n");
  SELrem(sel,vfd,200);
  if (!vic)
  {
    message("capture device fail, can't stop");
    return ;
  }
  vic= vica_start(vic);
  if (!vic)
    message("capture device fail 2, can't stop");
}

static int conn()
{
  if (IPFconnect("192.168.1.110",4598)) return datasock= -1;
  return datasock= IPFdataSOCK();
}

void motor_control(char x, char y)
{
  int i;
  uint8_t tbuf[4],rbuf[4];
  tbuf[0]= 0xc5; tbuf[1]= x; tbuf[2]= y; tbuf[3]= 0xc5+x+y;
  if (!SPIxfr(tbuf, rbuf, 4)) return ;
  for(i=0;i<3;i++)
   if (tbuf[i]!=rbuf[i+1]) 
   {
     fprintf(stderr,"SPI failed\n");
     fprintf(stderr,"TX: %02x %02x %02x %02x\n", 
                     tbuf[0], tbuf[1], tbuf[2], tbuf[3]);
     fprintf(stderr,"RX: %02x %02x %02x %02x\n", 
                     rbuf[0], rbuf[1], rbuf[2], rbuf[3]);
     break;
   }
}

void recycle_drive_enable()
{
  gpio_clr(0);
  usleep(10*1000);
  gpio_set(0);
  message("recycle_drive_enable");
}

int main (int argc, char **argv)
{
  int ev, try;
  unsigned char *inp; unsigned int inpl;

  vica_error_output= remoprint;
  sel= SELnew();

  if (conn()<0) { printf("can't connect\n"); return 1; }

  message("Hello server");
  
  vic= vica_init("/dev/video0","MJPG", 4,640,480);
  if (!vic) { message("can't open device /dev/video0"); return 1; }

  message("Device opened");

  vfd= vica_fd(vic);

  SELadd(sel, datasock, 100, SELr | SELx);

  gpio_output(0);
  if (SPIinit(argv[1],atoi(argv[2]))) return 1;

  gpio_set(0);

again:
  while(1)
  {
    for(ev=SELbeg(sel,-1);ev!=SELend;ev=SELnex(sel))
      switch(ev)
      {
      case SELEV(x, 100): goto reconnect;
      case SELEV(r, 100): if (IPFread()) goto reconnect; break;
      case SELEV(r, 200):
        if (!(vic=vica_get(vic,&inp,&inpl)))
        {
          message("error in capturing frame");
          SELrem(sel, vfd, 200);
          break;
        } 
        have_image(inpl, inp);
        vica_put(vic);
        break;
      }
  }

reconnect:
  printf("connection lost, reconnecting\n");
  SELrem(sel, datasock, 100);
  try= 1;
  while(conn()<0) { printf("retry connect %d\n", try++); sleep(5); }
  SELadd(sel, datasock, 100, SELr | SELx);
  goto again;   
}

