#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <stdint.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>
#include "spi-rpi.h"

static uint32_t spiSPEED;
static int spiFD=-1;

static int ERR(const char *fmt, ...)
{
  int eno;
  va_list ap;
  eno= errno;
  fprintf(stderr,"errno=%d (%s):", eno, strerror(eno));
  va_start(ap,fmt);
  vfprintf(stderr,fmt,ap);
  va_end(ap);
  if (spiFD>=0) { close(spiFD); spiFD= -1; }
  return -1;
}

int SPIinit(char *devname, uint32_t spd)
{
  uint8_t wmode,rmode;
  uint8_t wbits,rbits;
  uint32_t wspeed, rspeed; 

  spiFD= open(devname, O_RDWR);
  if (spiFD<0) return ERR("can't open device\n");
  
  wmode= SPI_NO_CS | SPI_CPHA;
  if (ioctl(spiFD, SPI_IOC_WR_MODE, &wmode)==-1)
     return ERR("can't set SPI mode\n");
  if (ioctl(spiFD, SPI_IOC_RD_MODE, &rmode)==-1)
     return ERR("can't get SPI mode\n");
  if (rmode!=wmode)
     return ERR("SPI mode written: %d, read: %d\n", wmode, rmode);

  wbits= 8;
  if (ioctl(spiFD, SPI_IOC_WR_BITS_PER_WORD, &wbits)==-1)
     return ERR("can't set SPI bits\n");
  if (ioctl(spiFD, SPI_IOC_RD_BITS_PER_WORD, &rbits)==-1)
     return ERR("can't get SPI bits\n");
  if (rbits!=wbits)
     return ERR("SPI bits written: %d, read: %d\n", wbits, rbits);

  wspeed= spd;
  if (ioctl(spiFD, SPI_IOC_WR_MAX_SPEED_HZ, &wspeed)==-1)
     return ERR("can't set SPI speed\n");
  if (ioctl(spiFD, SPI_IOC_RD_MAX_SPEED_HZ, &rspeed)==-1)
     return ERR("can't get SPI speed\n");
  printf("SPI speed written: %d, read: %d\n", wspeed, rspeed);
  spiSPEED= rspeed;
  return 0;
}

void SPIclose()
{
  if (spiFD>=0) { close(spiFD); spiFD= -1; }
}

int SPIxfr(unsigned char *Tbuf, unsigned char *Rbuf, int len)
{
  struct spi_ioc_transfer xfr;

  if (spiFD<0) return -1;
  if (len<=0) return 0;
  xfr.tx_buf= (uint64_t) (uintptr_t) Tbuf;
  xfr.rx_buf= (uint64_t) (uintptr_t) Rbuf;
  xfr.len= len;
  xfr.delay_usecs= 0; // I don't know what this does
  xfr.speed_hz= spiSPEED;
  xfr.bits_per_word= 8;
  xfr.cs_change= 0;

  xfr.tx_nbits= 8;
  xfr.rx_nbits= 8;
  xfr.pad= 0;

  if (ioctl(spiFD, SPI_IOC_MESSAGE(1), &xfr)<1)
    return ERR("SPI transfer failed\n");
  return 0;
}
