
#if (F_CPU/2/PWM_FREQ <= 65535)
#define PWM_PRESCALER  1
#define PWM_CLKSEL     1
#elif (F_CPU/2/PWM_FREQ/8 <= 65535)
#define PWM_PRESCALER  8
#define PWM_CLKSEL     2
#elif (F_CPU/2/PWM_FREQ/64 <= 65535)
#define PWM_PRESCALER  64
#define PWM_CLKSEL     3
#elif (F_CPU/2/PWM_FREQ/256 <= 65535)
#define PWM_PRESCALER  256
#define PWM_CLKSEL     4
#else 
#define PWM_PRESCALER  1024
#define PWM_CLKSEL     5
#endif

#define PWM_TOP    (F_CPU/2/PWM_FREQ/PWM_PRESCALER)
#if (PWM_TOP>65535 || PWM_TOP<2)
#error "Invalid PWM frequency, unable to compute prescaler and TOP"
#endif

static void pwm_init()
{
   // Disable pwm1, so that ICR etc. doesn't get garbled later.
   TCCR1A= 0;

   // WGM1_30 = 1010 for phase correct pwm, with TOP in ICR1

   // I set the control register B first, in order to 
   // make sure that the clock comes from the prescaler and
   // not some external event source
   TCCR1B= PWM_CLKSEL | _BV(WGM13);

   // We don't use interrupts at all, we simply clear the mask
   // register for that.
  TIMSK= 0;
   // This is the flags register for interrupts. I clear it just
   // for the sake of completeness.
  TIFR= 0;
   // Setting the fixed frequency
  ICR1H= PWM_TOP >> 8;
  ICR1L= PWM_TOP  & 0xff;

  // this actually has no effect since we haven't
  // started the pwm signals yet.
  OCR1AH= 0;
  OCR1AL= 0; 
  OCR1BH= 0;
  OCR1BL= 0;

  // reset the counter in case the timer was used before for some
  // other purpose
  TCNT1H= 0;
  TCNT1L= 0;
  
  // I will start the PWM later on, this simply selects the
  // rest of the bits for the PWM mode.
  TCCR1A= _BV(WGM11);
}

  // on match while counting up, output is cleared. on match 
  // while counting down, output is set. If you have 100 as 
  // TOP and 30 as OCR, then the output will remain high until 
  // the count reaches 30. After this, it will remain low
  // for 70 counts. Then the counter starts counting down.
  // The line still stays low for 70 cycles. Then it reaches
  // 30 again and the line goes high. This goes on for 30 
  // counts until the count reaches 0 again. In one cycle,
  // we had 60 counts high and 140 counts low on the line.
  // This of course is 30%.
static uint16_t duty_to_ocr(uint16_t V)
{
  uint32_t Z;
  Z= V;
  Z= (Z*PWM_TOP)>>16;
  return Z;
}

// dutyA and dutyB are 16 bit fractions. These values 
// are converted to proper OCR1A/OCR1B values using TOP 
static void pwm_duty(uint16_t dutyA, uint16_t dutyB)
{
  dutyA= duty_to_ocr(dutyA);
  dutyB= duty_to_ocr(dutyB);

  OCR1AH= dutyA>>8; OCR1AL= dutyA;
  OCR1BH= dutyB>>8; OCR1BL= dutyB;
}

static void pwm_start(uint16_t dutyA, uint16_t dutyB)
{
  pwm_duty(dutyA,dutyB);

  // Value 10 sets the non-inverting mode.
  TCCR1A |= _BV(COM1A1) | _BV(COM1B1);
}

void pwm_stop()
{
  TCCR1A &= ~(_BV(COM1A1) | _BV(COM1A0) | _BV(COM1B1) | _BV(COM1B0));
}
