#include "config-rx.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdint.h>
#include "ports.h"

#define RF24_RECEIVER
#include "rf24.c"
#include "pwm.c"
#include "uspi.c"

static void mlSTOP() { PORCLR(mlA); PORCLR(mlB); }
static void mrSTOP() { PORCLR(mrA); PORCLR(mrB); }

static void mlDIR(int8_t D)
{
  PORCLR(mlA); PORCLR(mlB);
  _delay_ms(1);
  if (D<0)
    PORSET(mlB);
  else if (D>0)
    PORSET(mlA);
}

static int32_t window(int32_t D, int32_t V, int32_t L, int32_t H)
{
  if (V<L) return L;
  if (V>H) return H;
  return V;
}

static void mrDIR(int8_t D)
{
  PORCLR(mrA); PORCLR(mrB);
  _delay_ms(1);
  if (D<0)
    PORSET(mrB);
  else if (D>0)
    PORSET(mrA);
}


#if 0
void leds_init()
{
  SETOUT(LED3); SETOUT(LED2); SETOUT(LED1); SETOUT(LED0);
  PORCLR(LED3); PORCLR(LED2); PORCLR(LED1); PORCLR(LED0);
}

void nibput(uint8_t K)
{
  if (K&8) PORSET(LED3); else PORCLR(LED3);
  if (K&4) PORSET(LED2); else PORCLR(LED2);
  if (K&2) PORSET(LED1); else PORCLR(LED1);
  if (K&1) PORSET(LED0); else PORCLR(LED0);
}


void leds_put(uint8_t K)
{
  nibput(K>>4);
  _delay_ms(2000);
  nibput(K);
}

uint8_t TRXreg(uint8_t K)
{
  TRXbegin(R_REGISTER|K);
  TRXput(0);
  TRXend();
  return TRXbuffer[1];
}
#endif

static int8_t dir;
static uint32_t slept;

void control_motors(uint8_t X, uint8_t Y)
{
  uint32_t L,R; int8_t Xdir, Ydir;
  slept= 0;
  if (Y&0x80) Ydir= -1; else Ydir= 1; Y &= ~0x80;
  if (X&0x80) Xdir= -1; else Xdir= 1; X &= ~0x80;

  if (!(X+Y)) { dir=0; mlSTOP(); mrSTOP(); return; }

  L= R= Y;
       
  L= window(L,L+Xdir*X, 0, 0xf);
  R= window(R,R-Xdir*X, 0, 0xf);
      
  if (Ydir!=dir) 
     { mlSTOP(); mrSTOP(); _delay_ms(5); mlDIR(Ydir); mrDIR(Ydir); }
  dir= Ydir;
  pwm_duty((L * 0xffFF)/15, (R * 0xffFF)/15);
}

volatile static uint8_t rpiX, rpiY;
volatile static uint32_t uspiR, uspiP;
void uspi_packet()
{
  uint8_t K;
  if (uspi_buffer[0]!=0xc5) return ;
  K= uspi_buffer[0]+uspi_buffer[1]+uspi_buffer[2];
  if (K!=uspi_buffer[3]) return ;
  rpiX= uspi_buffer[1];
  rpiY= uspi_buffer[2];
  uspiR++;
}


int main()
{
  uint8_t packet[3];
  uspi_init();
  sei();
  _delay_ms(20);
  rf24_init();

  pwm_init();

  SETOUT(mlPWM);  PORCLR(mlPWM); SETOUT(mlA);    PORCLR(mlA);
  SETOUT(mlB);    PORCLR(mlB);   SETOUT(mlEN);   PORCLR(mlEN);

  SETOUT(mrPWM);  PORCLR(mrPWM); SETOUT(mrA);    PORCLR(mrA);
  SETOUT(mrB);    PORCLR(mrB);   SETOUT(mrEN);   PORCLR(mrEN);

  pwm_start(0,0);

  PORSET(mrEN); PORSET(mlEN);

#if 0

  while(1)
  {
     mlDIR(1); mrDIR(1); pwm_duty(0xff00, 0xff00); _delay_ms(5000);
     mlSTOP(); mrSTOP(); _delay_ms(1000);
     mlDIR(0); mrDIR(0); _delay_ms(5000);
     mlSTOP(); mrSTOP(); _delay_ms(1000);
  }
#endif

  dir= 0;
  mlDIR(0); mrDIR(0);

  pwm_duty(0x0000, 0x0000);

  while(1)
  {
    _delay_ms(1);
    if (rf24_have_packet())
    {
      rf24_recv(packet,3);
      control_motors(packet[1], packet[2]);
      continue;
    }
    if (uspi_en && uspiR!=uspiP)
    {
      uspiP= uspiR;
      control_motors(rpiX, rpiY);
      continue;
    }
    slept++;
    if (slept>=100)
    {
      slept= 0;
      mlSTOP(); mrSTOP();
      dir= 0;
      pwm_duty(0,0);
    }
  }
}
