#include "config-tx.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdint.h>
#include "ports.h"

#define RF24_TRANSMITTER
#include "rf24.c"
#include "joy.c"

static uint8_t button;
static uint8_t X, Y;

static uint32_t Xmid=0x0200, Ymid=0x0200;
static uint16_t count;

uint8_t get_button()
{
  if (PINGET(BUTTON)) return 0;
               return 1;
}

uint8_t TTXreg(uint8_t K)
{
  rf24_begin(R_REGISTER|K);
  rf24_put(0);
  rf24_end();
  return rf24_buffer[1];
}


static uint8_t packet[3];


 // value is between 0x0000 - 0x03FF
 // I need 16 forward and 16 backward values.
 // the top bit will be direction bit.
static uint8_t joytransx(uint16_t V)
{
  uint16_t D;
  uint8_t S;
  if (V>=Xmid) { D= (V-Xmid); S= 0; }
  else         { D= (Xmid-V); S= 0x80; }
  D>>= 5;
  if (D>0xf) D= 0xf;
  return D | S;
}

static uint8_t joytransy(uint16_t V)
{
  uint16_t D;
  uint8_t S;
  V= 0x3ff-V;
  if (V>=Ymid) { D= (V-Ymid); S= 0; }
  else         { D= (Ymid-V); S= 0x80; }
  D>>= 5;
  if (D>0xf) D= 0xf;
  return D | S;
}

void calibrate()
{
  Xmid= joy_get(0);
  Ymid= 0x3ff-joy_get(1);
}

void blink(uint8_t n)
{
  uint8_t i;
  for(i=0;i<n;i++) 
    { PORSET(LED); _delay_ms(300); PORCLR(LED); _delay_ms(300); }
  PORSET(LED); _delay_ms(1000); PORCLR(LED); _delay_ms(300);
}

static void send_packet()
{
  packet[0]= button;
  packet[1]= X;
  packet[2]= Y;
  rf24_send(packet, 3); 
}

int main()
{
  joy_init();
  SETINP(BUTTON); PINPU0(BUTTON);
  SETOUT(LED); 
  _delay_ms(20);
  rf24_init();

  blink(1);

  while(1)
  {
    _delay_ms(10);
    button= get_button();
    if (button) count++;
           else count= 0;
    if (count==150)
    {
      count= 0;
      blink(3);
      calibrate();
      continue;
    }
    X= joytransx(joy_get(0));
    Y= joytransy(joy_get(1));
    send_packet();
  }
}
